# Generated by Django 3.0.5 on 2020-07-14 16:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_email', '0002_auto_20200528_1257'),
    ]

    operations = [
        migrations.AddField(
            model_name='emaillog',
            name='from_name',
            field=models.CharField(blank=True, help_text='Name of the sender', max_length=200, null=True),
        ),
    ]
